
#import "PGController.h"
#import "PowerGlove.h"

/* Internal functions */
void timedEntryHandler(DPSTimedEntry teNumber, double now, char *userData)
{
    id object = (id)userData;
    [object getPowerGloveData];
}

@implementation PGController

/* Action methods */
- showInfoPanel:sender
{
    if (infoPanel == nil) {
        if (![NXApp loadNibSection:"infopanel.nib" owner:self withNames:NO]) {
	    return nil;
	}
    }
    [infoPanel makeKeyAndOrderFront:nil];
    return self;
}

- showPreferencesPanel:sender
{
    if (preferencesPanel == nil) {
        if (![NXApp loadNibSection:"preferencespanel.nib" owner:self withNames:NO]) {
	    return nil;
	}
	/* Initialize sample delay settings */
        [sampleDelaySlider setMinValue:(double)0.001];
        [sampleDelaySlider setMaxValue:(double)0.5];
        [sampleDelaySlider setFloatValue:sampleDelay];
        [sampleDelayField setFloatValue:sampleDelay];
    }
    [preferencesPanel makeKeyAndOrderFront:nil];
    return self;
}

- setSampleDelay:sender
{
    sampleDelay = [sender floatValue];
    [sampleDelayField setFloatValue:sampleDelay];
    
    /* Replace old timed entry with a new one */
    if (powerGloveTE) {
        DPSRemoveTimedEntry(powerGloveTE);
    }
    powerGloveTE = DPSAddTimedEntry(sampleDelay,
                                   (DPSTimedEntryProc)timedEntryHandler,
				   self, NX_BASETHRESHOLD);
    return self;
}

/* Misc. methods */
- getPowerGloveData
{
    /* Previous button values */
    static BOOL oldAPressed = NO;
    static BOOL oldBPressed = NO;
    static BOOL oldStartPressed = NO;
    static BOOL oldSelectPressed = NO;
    static BOOL oldPadUp = NO;
    static BOOL oldPadDown = NO;
    static BOOL oldPadLeft = NO;
    static BOOL oldPadRight = NO;
 
    /* Previous finger values */
    static int oldRing = 0;
    static int oldMiddle = 0;
    static int oldIndex = 0;
    static int oldThumb = 0;
    
    /* Previous position and rotation values */
    static int oldGloveX = 0;
    static int oldGloveY = 0;
    static int oldGloveZ = 0;
    static int oldGloveRot = 0;

    BOOL tempBool;
    BOOL changed = NO;
    int tempInt;
    
    /* Update buttons if they change */
    if ((tempBool = [powerGlove aPressed]) != oldAPressed) {
        [aButton setState:(oldAPressed = tempBool)];
	changed = YES;
    }
    if ((tempBool = [powerGlove bPressed]) != oldBPressed) {
        [bButton setState:(oldBPressed = tempBool)];
	changed = YES;
    }
    if ((tempBool = [powerGlove startPressed]) != oldStartPressed) {
        [startButton setState:(oldStartPressed = tempBool)];
	changed = YES;
    }
    if ((tempBool = [powerGlove selectPressed]) != oldSelectPressed) {
        [selectButton setState:(oldSelectPressed = tempBool)];
	changed = YES;
    }
    if ((tempBool = [powerGlove padUp]) != oldPadUp) {
        [upButton setState:(oldPadUp = tempBool)];
	changed = YES;
    }
    if ((tempBool = [powerGlove padDown]) != oldPadDown) {
        [downButton setState:(oldPadDown = tempBool)];
	changed = YES;
    }
    if ((tempBool = [powerGlove padLeft]) != oldPadLeft) {
        [leftButton setState:(oldPadLeft = tempBool)];
	changed = YES;
    }
    if ((tempBool = [powerGlove padRight]) != oldPadRight) {
        [rightButton setState:(oldPadRight = tempBool)];
	changed = YES;
    }

    /* Update finger value form fields */
    if ((tempInt = [powerGlove ring]) != oldRing) {
        [ringSlider setIntValue:(oldRing = tempInt)];
	changed = YES;
    }
    if ((tempInt = [powerGlove middle]) != oldMiddle) {
        [middleSlider setIntValue:(oldMiddle = tempInt)];
	changed = YES;
    }
    if ((tempInt = [powerGlove index]) != oldIndex) {
        [indexSlider setIntValue:(oldIndex = tempInt)];
	changed = YES;
    }
    if ((tempInt = [powerGlove thumb]) != oldThumb) {
        [thumbSlider setIntValue:(oldThumb = tempInt)];
	changed = YES;
    }
        
    /* Update position and rotation form fields */
    if ((tempInt = [powerGlove gloveX]) != oldGloveX) {
        [xField setIntValue:(oldGloveX = tempInt)];
	changed = YES;
    }
    if ((tempInt = [powerGlove gloveY]) != oldGloveY) {
        [yField setIntValue:(oldGloveY = tempInt)];
	changed = YES;
    }
    if ((tempInt = [powerGlove gloveZ]) != oldGloveZ) {
        [zField setIntValue:(oldGloveZ = tempInt)];
	changed = YES;
    }
    if ((tempInt = [powerGlove gloveRot]) != oldGloveRot) {
        [rollField setIntValue:(int)(oldGloveRot = tempInt)*30];
	changed = YES;
    }

    /* Redraw window only if something changed */
    if (changed) {
        [powerGloveView display];

        /* Wait for Window Server to finish drawing everything */
        NXPing();
    }

    return self;
}

@end

/* Delegate methods */
@implementation PGController(ApplicationDelegate)

- appDidInit:sender
{
    /* Initialize the PowerGlove */
    [powerGlove init];
    [powerGlove setxHyst:0 yHyst:0 zHyst:0];
    
    /* Set up timed entry to get glove data periodically */
    sampleDelay = 0.1;
    powerGloveTE = DPSAddTimedEntry(sampleDelay,
                                   (DPSTimedEntryProc)timedEntryHandler,
				   self, NX_BASETHRESHOLD);
    return self;
}

@end
